@@ -5,7 +5,7 @@
     Copyright (C) 1997-2000 Peter Putzer
                             putzer@kde.org
 
-	$Id: TopWidget.cpp,v 1.102 2001/05/23 03:45:32 jsinger Exp $
+	$Id: TopWidget.cpp,v 1.117 2003/10/07 17:08:16 aacid Exp $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of version 2 of the GNU General Public License
@@ -18,7 +18,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 /****************************************************************
@@ -30,12 +30,12 @@
 
 #include <ctype.h>
 
-#include <qpopmenu.h>
+#include <qpopupmenu.h>
 #include <qkeycode.h>
-#include <qmsgbox.h>
+#include <qmessagebox.h>
 #include <qfontmetrics.h>
 #include <qpainter.h>
-#include <qmultilinedit.h>
+#include <qmultilineedit.h>
 #include <qdatetime.h>
 #include <kprinter.h>
 #include <qpaintdevicemetrics.h>
@@ -60,7 +60,7 @@
 #include <kaction.h>
 #include <kstdaction.h>
 #include <kdebug.h>
-#include <kapp.h>
+#include <kapplication.h>
 #include <kglobal.h>
 #include <khelpmenu.h>
 #include <klocale.h>
@@ -86,13 +86,15 @@
 #include "OldView.h"
 #include "IOCore.h"
 #include "TopWidget.h"
+#include <qlabel.h>
+#include <kstatusbar.h>
 
 namespace Status
 {
   enum {
     Changed, Checklist, Writable
   };
-}; // namespace Status
+} // namespace Status
 
 KSVTopLevel::KSVTopLevel()
   : KMainWindow(0, 0L, WStyle_ContextHelp|WDestructiveClose),
@@ -132,7 +134,7 @@ KSVTopLevel::KSVTopLevel()
   kapp->setMainWidget (this);
 
   setCaption(false);
-  
+
   initActions(); // order dependency
   mView = new KSVContent (mOpenWith->popupMenu(), this, "Content"); // order dependency
   initTools(); // order dependency
@@ -193,7 +195,7 @@ void KSVTopLevel::initTools()
   connect (mStopDlg, SIGNAL(doAction(const QString&)),
 		   mView, SLOT(editService(const QString&)));
   connect (mRestartDlg, SIGNAL(doAction(const QString&)),
-		   mView, SLOT(restartService(const QString&))); 
+		   mView, SLOT(restartService(const QString&)));
   connect (mEditDlg, SIGNAL(doAction(const QString&)),
 		   mView, SLOT(editService(const QString&)));
 
@@ -226,7 +228,7 @@ void KSVTopLevel::initActions ()
 
   mFileSaveLog = KStdAction::save (this, SLOT(slotSaveLog()), coll, "ksysv_save_log");
   mFileSaveLog->setText (i18n("Save &Log..."));
-  mFileSaveLog->setAccel (Key_L+CTRL);
+  mFileSaveLog->setShortcut (Key_L+CTRL);
   mFileSaveLog->setEnabled (false);
 
   // disabled due to complexity
@@ -237,7 +239,7 @@ void KSVTopLevel::initActions ()
   mFilePrintLog->setEnabled (false);
 
   mFileQuit = KStdAction::quit(this, SLOT(close()), coll);
-  
+
   // setup Edit menu
   mEditUndo = KStdAction::undo(this, SLOT(editUndo()), coll);
   mEditUndo->setEnabled (false);
@@ -278,7 +280,7 @@ void KSVTopLevel::initActions ()
   mToolsStopService = new KToggleAction (i18n("&Stop Service..."), "ksysv_stop", 0,
                                          mStopDlg, SLOT (toggle()),
                                          coll, "ksysv_stop_service");
-  
+
   mToolsRestartService = new KToggleAction (i18n("&Restart Service..."), 0,
                                             mRestartDlg, SLOT (toggle()),
                                             coll, "ksysv_restart_service");
@@ -315,7 +317,7 @@ bool KSVTopLevel::queryExit() {
 
 void KSVTopLevel::slotClearChanges()
 {
-  if (mChanged && 
+  if (mChanged &&
       KMessageBox::Yes ==
       KMessageBox::questionYesNo(kapp->mainWidget(),
 				 i18n("Do you really want to revert all unsaved changes?"),
@@ -332,8 +334,8 @@ void KSVTopLevel::slotAcceptChanges() {
   if (KMessageBox::Yes ==
       KMessageBox::warningYesNo(kapp->mainWidget(),
 				i18n("You're about to save the changes made to your init "
-				     "configuration.\nPlease realize that wrong settings can "
-				     "make your system hang on startup.\n\n"
+				     "configuration. Wrong settings can "
+				     "make your system hang on startup.\n"
 				     "Do you wish to continue?"),
 				i18n("Save Configuration"),
 				i18n("&Save"),
@@ -410,14 +412,14 @@ void KSVTopLevel::initStatusBar()
                                  "when you use the <strong>Save Options command</strong>.</p>"));
   QToolTip::add (visBox, i18n ("Show only the selected runlevels"));
 
-  new QLabel (i18n(" Show Runlevels: "), visBox, "visLabel");
+  new QLabel (i18n("Show runlevels:"), visBox, "visLabel");
   for (int i = 0; i < ksv::runlevelNumber; ++i)
 	{
 	  QString label; label.setNum (i);
 
 	  mVisible[i] = new QCheckBox (label, visBox, label.latin1());
 	  mVisible[i]->setChecked (mConfig->showRunlevel (i));
-	  
+
 	  group->insert (mVisible[i]);
 	}
 
@@ -443,7 +445,7 @@ void KSVTopLevel::initStatusBar()
   status->addWidget (authIconBox, 0, false);
   status->insertItem ("", Status::Changed, 100);
   status->addWidget (visBox, 0, true);
-  
+
   status->setItemAlignment (Status::Changed, AlignLeft|AlignVCenter);
 }
 
@@ -452,7 +454,7 @@ void KSVTopLevel::slotShowConfig()
   if (!mPreferences)
     {
       mPreferences = KSVPreferences::self();
-      
+
       connect (mPreferences, SIGNAL (updateColors ()),
                this, SLOT (updateColors ()));
 
@@ -528,10 +530,10 @@ void KSVTopLevel::editCut() {
 	{
 	  KSVDrag* mime = new KSVDrag (*list->currentItem()->data(), 0L, 0L);
 	  kapp->clipboard()->setData (mime);
-	  
+
 	  KSVData data = *list->currentItem()->data();
 	  delete list->currentItem();
-		  
+
 	  mUndoList->push (new RemoveAction (list, &data));
 	  setChanged (true);
 	}
@@ -551,11 +553,11 @@ void KSVTopLevel::editCopy()
 void KSVTopLevel::editPaste()
 {
   KSVDragList* list = mView->getOrigin();
-  
+
   if (list)
     {
 	  KSVData data;
-	  
+
 	  if (KSVDrag::decodeNative (kapp->clipboard()->data(), data))
 		{
 		  KSVAction* action = 0L;
@@ -616,9 +618,9 @@ void KSVTopLevel::editUndo ()
 void KSVTopLevel::editRedo ()
 {
   KSVAction* action = mRedoList->top();
-  
+
   mRedoList->redoLast();
-  
+
   setChanged (true);
   mUndoList->push (action);
 }
@@ -632,9 +634,9 @@ void KSVTopLevel::setPaste (bool val)
 void KSVTopLevel::setLog (bool val)
 {
   mConfig->setShowLog(val);
-  
+
   mOptionsToggleLog->setChecked (val);
-  
+
   mView->setDisplayScriptOutput(val);
 }
 
@@ -676,7 +678,7 @@ void KSVTopLevel::print()
 
 //   static QPrintDialog* dlg = new QPrintDialog (&prt, this, "KSysV Print Dialog");
 //   dlg->setCaption(kapp->makeStdCaption (i18n("Print")));
-  
+
 //   if (dlg->exec() == QDialog::Accepted)
 //     {
 //       int y = 10;
@@ -684,7 +686,7 @@ void KSVTopLevel::print()
 //       p.begin( &prt );
 
 //       QPaintDeviceMetrics metrics (&prt);
-  
+
 //       p.setFont (QFont("courier", 20, QFont::Bold));
 //       QFontMetrics fm = p.fontMetrics();
 
@@ -700,17 +702,17 @@ void KSVTopLevel::print()
 //         {
 //           p.setFont (QFont("courier", 16, QFont::Bold));
 //           QFontMetrics fm = p.fontMetrics();
-          
+
 //           p.drawText (10, y, i18n ("Runlevel %1").arg(i));
 //           y += fm.lineSpacing();
 
 //           checkPage
-          
+
 //           p.drawText (10, y, i18n ("Started Services"));
 //           y += fm.lineSpacing() * 2; // an extra empty line
 
 //        checkPage
-            
+
 //           p.setFont (QFont("courier", 10));
 //           fm = p.fontMetrics();
 
@@ -733,9 +735,9 @@ void KSVTopLevel::print()
 //           y += fm.lineSpacing(); // an extra empty line
 //           p.drawText (10, y, i18n ("Stopped Services"));
 //           y += fm.lineSpacing() * 2; // an extra empty line
-   
+
 //        checkPage
-       
+
 //           p.setFont (QFont("courier", 10));
 //           fm = p.fontMetrics();
 
@@ -762,7 +764,7 @@ void KSVTopLevel::print()
 //       //           ++it)
 //       //         {
 //       //           y += fm.ascent();
-          
+
 //       //           QString line = *it;
 //       //           if (line.isNull())
 //       //             line = " ";
@@ -770,29 +772,29 @@ void KSVTopLevel::print()
 //       //           line.replace( QRegExp("\t"), "        " );
 
 //       //           strncpy(buf,line.data(),160);
-          
+
 //       //           for (int j = 0 ; j <150; j++)
 //       //             {
 //       //               if (!isprint(buf[j]))
 //       //                 buf[j] = ' ';
 //       //             }
-          
+
 //       //           buf[line.length()] = '\0';
 //       //           p.drawText( 10, y, buf );
 //       //           y += fm.descent();
 //       //         }
-      
-//       p.end();        
+
+//       p.end();
 //     }
 }
 
 void KSVTopLevel::printLog()
-{  
+{
   KPrinter *prt = new KPrinter();
   prt->setDocName(kapp->aboutData()->programName() + " Log File");
   prt->setCreator(kapp->aboutData()->programName());
-  
-  if (prt->setup(this))
+
+  if (prt->setup(this, i18n("Print Log File")))
     {
       KBusyManager::self()->setBusy (true);
 
@@ -800,48 +802,48 @@ void KSVTopLevel::printLog()
       QPaintDeviceMetrics pm (prt);
       const int height = pm.height();
       const int width = pm.width();
-      
+
       QStringList lines = QStringList::split ('\n', mView->xmlLog(), true);
       QStringList::Iterator line = lines.begin();
 
       int y = 0;
-      
+
       {
-        QSimpleRichText rheading (i18n("<h1>KDE Sys-V Init Editor Log</h1>"), 
+        QSimpleRichText rheading (i18n("<h1>KDE Sys-V Init Editor Log</h1>"),
                                   QFont("times"), QString::null, ksv::styleSheet(),
                                   QMimeSourceFactory::defaultFactory());
         rheading.setWidth (&p, width);
         int tmp_h = rheading.height();
         QRegion region (0, y, width, tmp_h);
-        rheading.draw (&p, 0, y, region, palette(), 0L);
+        rheading.draw (&p, 0, y, region, colorGroup(), 0L);
 
         y += tmp_h;
-        
-        QSimpleRichText rdate (i18n("<h3>Printed on %1</h3><br/><br/>").arg(QDateTime::currentDateTime().toString()), 
+
+        QSimpleRichText rdate (i18n("<h3>Printed on %1</h3><br/><br/>").arg(KGlobal::locale()->formatDateTime(QDateTime::currentDateTime())),
                                QFont("times"), QString::null, ksv::styleSheet(),
                                QMimeSourceFactory::defaultFactory());
         rdate.setWidth (&p, width);
         tmp_h = rdate.height();
-	QRegion r2 (0, y, width, tmp_h); 
-        rdate.draw (&p, 0, y, r2, palette(), 0L);
+	QRegion r2 (0, y, width, tmp_h);
+        rdate.draw (&p, 0, y, r2, colorGroup(), 0L);
 
         y += tmp_h;
       }
 
       while (line != lines.end())
-        {         
+        {
           while (y < height)
             {
               QSimpleRichText rich (*line, QFont("times"), QString::null, ksv::styleSheet(),
                                     QMimeSourceFactory::defaultFactory());
               rich.setWidth (&p, width);
-              
+
               int tmp_h = rich.height();
               if (y + tmp_h > height)
                 break;
-              
+
               QRegion region (0, y, width, tmp_h);
-              rich.draw (&p, 0, y, region, palette(), 0L);
+              rich.draw (&p, 0, y, region, colorGroup(), 0L);
 
               y += tmp_h;
               ++line;
@@ -851,16 +853,17 @@ void KSVTopLevel::printLog()
 
               kapp->processEvents();
             }
-          
+
           prt->newPage();
           y = 0;
         }
 
     printing_finished:
-         
+
       p.end();
-      
+
       KBusyManager::self()->restore();
+      delete prt;
     }
 }
 
@@ -868,18 +871,18 @@ void KSVTopLevel::catchCannotGenerateNumber()
 {
   if (mConfig->showMessage (ksv::CouldNotGenerateSortingNumber)) {
     KMessageBox::information (kapp->mainWidget(),
-                              i18n ("<p>Sorry, I could not generate a valid " \
+                              i18n ("<p>Unable to generate a valid " \
                                     "sorting number for this position. This means " \
-                                    "that there was no number available in between "\
+                                    "that there was no number available between "\
                                     "the two adjacent services, and the service " \
                                     "did not fit in lexically.</p>" \
                                     "<p>Please adjust the sorting numbers manually "\
                                     "via the <strong>Properties dialog box</strong>.</p>"),
-                              i18n ("Could Not Generate Sorting Number"),
+                              i18n ("Unable to Generate Sorting Number"),
                               ksv::notifications[ksv::CouldNotGenerateSortingNumber]);
   }
 
-  statusBar()->message (i18n ("Could not generate sorting number. Please change manually."),
+  statusBar()->message (i18n ("Unable to generate sorting number. Please change manually."),
                         5000);
 }
 
@@ -923,7 +926,7 @@ void KSVTopLevel::dispatchEdit ()
       mEditCut->setEnabled (false);
       mEditProperties->setEnabled (false);
 	}
-  
+
   if (mView->scripts == list)
 	{
       mEditCut->setEnabled (false);
@@ -1003,7 +1006,7 @@ KActionCollection* KSVTopLevel::filteredActions ()
   // remove unwanted (internal) actions
   static KActionCollection coll = *actionCollection();
   static bool initialized = false;
-  
+
   if (!initialized)
     {
       coll.take(mPasteAppend);
@@ -1016,17 +1019,20 @@ KActionCollection* KSVTopLevel::filteredActions ()
 
 void KSVTopLevel::configureKeys ()
 {
-  KKeyDialog::configureKeys (filteredActions(), xmlFile(), true, this);
+  KKeyDialog::configure (filteredActions(), this, true);
 }
 
 void KSVTopLevel::configureToolbars ()
 {
-  KEditToolbar* dlg = new KEditToolbar (filteredActions(), xmlFile(), true, this);
-  
-  if (dlg->exec())
-    createGUI (xmlFile());
-  
-  delete dlg;
+  KEditToolbar dlg(filteredActions(), xmlFile(), true, this);
+  connect(&dlg, SIGNAL( newToolbarConfig() ), this, SLOT( slotNewToolbarConfig() ));
+  dlg.exec();
+}
+
+void KSVTopLevel::slotNewToolbarConfig()
+{
+  createGUI (xmlFile());
+  setTools(mConfig->showTools());
 }
 
 void KSVTopLevel::dispatchEditService (bool val)
@@ -1099,12 +1105,12 @@ void KSVTopLevel::saveAs ()
     }
 
   file.close();
-  
+
   KBusyManager::self()->restore();
 }
 
 void KSVTopLevel::load ()
-{  
+{
   static const QString& filter = ksv::nativeFileFilter();
   QString filename = KFileDialog::getOpenFileName(0L, filter, this);
 
@@ -1118,7 +1124,7 @@ void KSVTopLevel::load ()
   QDataStream s (&file);
   QValueList<KSVData>* startLists = new QValueList<KSVData>[ksv::runlevelNumber];
   QValueList<KSVData>* stopLists = new QValueList<KSVData>[ksv::runlevelNumber];
-  
+
   if (ksv::IO::loadSavedConfiguration (s, startLists, stopLists))
     {
       statusBar()->message(i18n("Configuration package loaded successfully."), 5000);
