@@ -1,3 +1,22 @@
+/*
+ *  Copyright (c) 1998 Denis Perchine <dyp@perchine.com>
+ *  Maintained by Adriaan de Groot <groot@kde.org>
+ *
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU General Public
+ *  License version 2 as published by the Free Software Foundation.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public License
+ *  along with this library; see the file COPYING.LIB.  If not, write to
+ *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ *  Boston, MA 02111-1307, USA.
+ **/
+
 #include "globals.h"
 
 #include <errno.h>
@@ -36,8 +55,9 @@
 #include "kglobal_.h"
 #include "kuser.h"
 #include "misc.h"
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 #include <kmessagebox.h>
+#include <kprocess.h>
 #include "editDefaults.h"	
 
 #ifdef _KU_QUOTA
@@ -55,7 +75,7 @@
 
 // class KUser
 
-KUser::KUser() : p_pwd("*") {
+KUser::KUser() : p_pwd(QString::fromLatin1("*")) {
 #if defined(__FreeBSD__) || defined(__bsdi__)
   p_change = 0;
   p_expire = 0;
@@ -76,6 +96,8 @@ KUser::KUser() : p_pwd("*") {
   isCreateHome = false;
   isCreateMailBox = false;
   isCopySkel = false;
+  isDeleteHome = false;
+  isDeleteMailBox = false;
 }
   
 KUser::KUser(const KUser *user) {
@@ -134,43 +156,43 @@ bool KUser::getCopySkel() {
   return isCopySkel;
 }
 
-const QString &KUser::getName() const {
+QString KUser::getName() const {
   return p_name;
 }
 
-const QString &KUser::getPwd() const {
+QString KUser::getPwd() const {
   return p_pwd;
 }
 
-const QString &KUser::getHomeDir() const {
+QString KUser::getHomeDir() const {
   return p_dir;
 }
 
-const QString &KUser::getShell() const {
+QString KUser::getShell() const {
   return p_shell;
 }
 
-const QString &KUser::getFullName() const {
+QString KUser::getFullName() const {
   return p_fname;
 }
 
 #if defined(__FreeBSD__) || defined(__bsdi__)
 // FreeBSD apparently uses the GECOS fields differently than other Unices.
 // Create some better named functions to make the FreeBSD code clear
-const QString &KUser::getOffice() const {
+QString KUser::getOffice() const {
   return p_office;
 }
 
-const QString &KUser::getWorkPhone() const {
+QString KUser::getWorkPhone() const {
   return p_ophone;
 }
 
-const QString &KUser::getHomePhone() const {
+QString KUser::getHomePhone() const {
   return p_hphone;
 }
 
 // New fields needed for the FreeBSD /etc/master.passwd file
-const QString &KUser::getClass() const {
+QString KUser::getClass() const {
   return p_class;
 }
 
@@ -178,21 +200,17 @@ time_t KUser::getLastChange() const {
   return p_change;
 }
 
-time_t KUser::getExpire() const {
-  return p_expire;
-}
-
 #else
 
-const QString &KUser::getOffice1() const {
+QString KUser::getOffice1() const {
   return p_office1;
 }
 
-const QString &KUser::getOffice2() const {
+QString KUser::getOffice2() const {
   return p_office2;
 }
 
-const QString &KUser::getAddress() const {
+QString KUser::getAddress() const {
   return p_address;
 }
 
@@ -207,7 +225,7 @@ gid_t KUser::getGID() const {
 }
 
 #ifdef HAVE_SHADOW
-const QString &KUser::getSPwd() const {
+QString KUser::getSPwd() const {
   return s_pwd;
 }
 
@@ -231,15 +249,36 @@ int KUser::getInactive() const {
   return s_inact;
 }
 
-int KUser::getExpire() const {
-  return s_expire;
-}
 
 int KUser::getFlag() const {
   return s_flag;
 }
 #endif
 
+/**
+* Return the number of days from the epoch until the
+* password expires.
+*/
+int KUser::getExpire() const 
+{
+#if defined(__FreeBSD__) || defined(__bsdi__)
+  time_t secondsToExpiry = p_expire;
+  QDate epoch(1970,1,1);
+  QDateTime expiryDate;
+  
+  expiryDate.setTime_t(secondsToExpiry);
+
+  return epoch.daysTo(expiryDate.date());
+#elif defined(HAVE_SHADOW) 
+  return s_expire;
+#else
+  return 0;
+#endif
+}
+
+
+
+
 void KUser::setName(const QString &data) {
   p_name = data;
 }
@@ -372,8 +411,8 @@ KUsers::KUsers() {
   sdw_uid = 0;
   sdw_gid = 0;
 
-  u.setAutoDelete(TRUE);
-  du.setAutoDelete(TRUE);
+  allUsers.setAutoDelete(TRUE);
+  usersToDelete.setAutoDelete(TRUE);
 
   if (!load())
     err->display();
@@ -386,13 +425,11 @@ void KUsers::fillGecos(KUser *user, const char *gecos) {
   // At least one part of the string exists
   for(;;) {
     pos = strchr(s, ',');
-    char val[200];
+    QString val;
     if(pos == NULL)
-      strcpy(val, s);
-    else {
-      strncpy(val, s, (int)(pos-s));
-      val[(int)(pos-s)] = 0;
-    }
+      val = QString::fromLocal8Bit(s);
+    else
+      val = QString::fromLocal8Bit(s, (int)(pos-s));
 
     switch(no) {
       case 0: user->setFullName(val); break;
@@ -462,10 +499,18 @@ bool KUsers::loadpwd() {
 
   // Start reading passwd file(s)
 
+#ifdef Q_OS_FREEBSD
+  // For FreeBSD, some weird effect in the C++ libraries
+  // eats the first entry of the passwd file on -CURRENT,
+  // which renders KUser a system-destroyer instead of
+  // just a useless tool.
+  setpwent();
+#endif
+
   for(int i = 0; i < MAXFILES; i++) {
     rc = stat(QFile::encodeName(filename), &st);		
     if(rc != 0) {						
-      err->addMsg(i18n("stat call on file %1 failed: %2\nCheck KUser Settings (Sources)\n").arg(filename).arg(strerror(errno)));	
+      err->addMsg(i18n("stat call on file %1 failed: %2\nCheck KUser Settings (Sources)\n").arg(filename).arg(QString::fromLocal8Bit(strerror(errno))));	
       err->display();						
       if( (processing_file & PASSWD) != 0 ) {			
         passwd_errno = errno;					
@@ -506,19 +551,19 @@ bool KUsers::loadpwd() {
       tmpKU = new KUser();
       tmpKU->setUID(p->pw_uid);
       tmpKU->setGID(p->pw_gid);
-      tmpKU->setName(p->pw_name);
-      tmpKU->setPwd(p->pw_passwd);
-      tmpKU->setHomeDir(p->pw_dir);
-      tmpKU->setShell(p->pw_shell);
+      tmpKU->setName(QString::fromLocal8Bit(p->pw_name));
+      tmpKU->setPwd(QString::fromLocal8Bit(p->pw_passwd));
+      tmpKU->setHomeDir(QString::fromLocal8Bit(p->pw_dir));
+      tmpKU->setShell(QString::fromLocal8Bit(p->pw_shell));
 #if defined(__FreeBSD__) || defined(__bsdi__)
-      tmpKU->setClass(p->pw_class);
+      tmpKU->setClass(QString::fromLatin1(p->pw_class));
       tmpKU->setLastChange(p->pw_change);
       tmpKU->setExpire(p->pw_expire);
 #endif
 
       if ((p->pw_gecos != 0) && (p->pw_gecos[0] != 0)) 
         fillGecos(tmpKU, p->pw_gecos);
-      u.append(tmpKU);
+      allUsers.append(tmpKU);
     }
 
     // End reading passwd_filename				
@@ -575,14 +620,14 @@ bool KUsers::loadsdw() {
   setspent();
 
   while ((spw = getspent())) {     // read a shadow password structure
-    if ((up = lookup(spw->sp_namp)) == NULL) {
+    if ((up = lookup(QString::fromLocal8Bit(spw->sp_namp))) == NULL) {
       err->addMsg(i18n("No /etc/passwd entry for %1.\nEntry will be removed at the next `Save'-operation.")
-		  .arg(spw->sp_namp));
+		  .arg(QString::fromLocal8Bit(spw->sp_namp)));
       err->display();
       continue;
     }
 
-    up->setSPwd(spw->sp_pwdp);        // cp the encrypted pwd
+    up->setSPwd(QString::fromLocal8Bit(spw->sp_pwdp));        // cp the encrypted pwd
     up->setLastChange(spw->sp_lstchg);
     up->setMin(spw->sp_min);
     up->setMax(spw->sp_max);
@@ -616,8 +661,11 @@ bool KUsers::save() {
 }
 
 bool KUsers::doCreate() {
-  for (unsigned int i=0; i<u.count(); i++) {
-    KUser *user = u.at(i);
+
+  QString h_dir;
+
+  for (unsigned int i=0; i<allUsers.count(); i++) {
+    KUser *user = allUsers.at(i);
 
     if(user->getCreateMailBox()) {
       user->createMailBox();
@@ -625,32 +673,42 @@ bool KUsers::doCreate() {
     }
 
     if(user->getCreateHome()) {
-       user->createHome();
-       user->setCreateHome(false);
+       if(user->createHome()) {
+           user->setCreateHome(false);
+           if(!user->getCopySkel())
+       	       user->createKDE();
+       }
+       else
+	  continue;	// if createHome fails, copySkel is irrelevant!
     }
 
     if(user->getCopySkel()) {
-       user->copySkel();
-       user->setCopySkel(false);
-       user->createKDE();
+       if((user->copySkel()) == 0) {
+          user->setCopySkel(false);
+       	  h_dir = user->getHomeDir();
+          if(!user->findKDE(h_dir))
+             user->createKDE();
+       }
     }
-  }
 
+  }
   return TRUE;
 }
 
 bool KUsers::doDelete() {
-	uint ucnt = du.count();
+	uint ucnt = usersToDelete.count();
   KUser *user;
 
-  user = du.first();
+  user = usersToDelete.first();
   for (unsigned int i=0; i<ucnt; i++) {
-    user = du.current(); 
-    user->removeHome();
+    user = usersToDelete.current();
+    if (user->isDeleteHome)
+      user->removeHome();
     user->removeCrontabs();
-    user->removeMailBox();
+    if (user->isDeleteMailBox)
+      user->removeMailBox();
     user->removeProcesses();
-    du.remove();
+    usersToDelete.remove();
   }
 
   return TRUE;
@@ -671,9 +729,7 @@ bool KUsers::savepwd() {
   QString nispasswd_filename;
   QString qs_minuid;			
 
-  const char *c_minuid;			
-  const char *pw_filename;		
-  const char *pn_filename;		
+
   char errors_found = '\0';		
     #define NOMINUID    0x01		
     #define NONISPASSWD 0x02		
@@ -683,25 +739,27 @@ bool KUsers::savepwd() {
   config->setGroup("sources");		
   passwd_filename = config->readEntry("passwdsrc");		
   nispasswd_filename = config->readEntry("nispasswdsrc");	
-  qs_minuid = config->readEntry("nisminuid");			
+  qs_minuid = config->readEntry("nisminuid");
+  QCString tmp1 = QFile::encodeName(passwd_filename);
+  const char *pw_filename = tmp1.data();   
+  QCString tmp2 = QFile::encodeName(nispasswd_filename);
+  const char *pn_filename = tmp2.data();
+
   if( (!qs_minuid.isEmpty()) && (nispasswd_filename != passwd_filename) ) {
-    c_minuid = qs_minuid.latin1();				
-    minuid = atoi(c_minuid);		
+    minuid = atoi(qs_minuid.latin1());		
   }
 
   // Backup file(s)			
 
   if(!passwd_filename.isEmpty()) {	
     if (!pw_backuped) {			
-      pw_filename = passwd_filename.latin1();   
-      backup(pw_filename);		
+      backup(passwd_filename);
       pw_backuped = TRUE;		
     }
   }
   if(!nispasswd_filename.isEmpty() && (nispasswd_filename != passwd_filename)){	
     if (!pn_backuped) {			
-      pn_filename = nispasswd_filename.latin1();   
-      backup(pn_filename);		
+      backup(nispasswd_filename);
       pn_backuped = TRUE;		
     }
   }
@@ -709,23 +767,23 @@ bool KUsers::savepwd() {
   // Open file(s)			
 
   if(!passwd_filename.isEmpty()) {	
-    if ((passwd_fd = fopen(QFile::encodeName(passwd_filename),"w")) == NULL) 
+    if ((passwd_fd = fopen(pw_filename,"w")) == NULL) 
       err->addMsg(i18n("Error opening %1 for writing").arg(passwd_filename));
   }					
 
   if(!nispasswd_filename.isEmpty() && (nispasswd_filename != passwd_filename)){	
-    if ((nispasswd_fd = fopen(QFile::encodeName(nispasswd_filename),"w")) == NULL) 
+    if ((nispasswd_fd = fopen(pn_filename,"w")) == NULL) 
       err->addMsg(i18n("Error opening %1 for writing").arg(nispasswd_filename));
   }					
 
     umask(0077);
 
-    for (unsigned int i=0; i<u.count(); i++) {
-      KUser *user = u.at(i);
+    for (unsigned int i=0; i<allUsers.count(); i++) {
+      KUser *user = allUsers.at(i);
       tmp_uid = user->getUID();	 	
 
 #if defined(__FreeBSD__) || defined(__bsdi__)
-      s = QString("%1:%2:%3:%4:%5:%6:%7:")
+      s = QString::fromLatin1("%1:%2:%3:%4:%5:%6:%7:")
         .arg(user->getName())
         .arg(user->getPwd())
 	.arg(user->getUID())
@@ -734,20 +792,20 @@ bool KUsers::savepwd() {
         .arg(user->getLastChange())
 	.arg(user->getExpire());
 
-      s1 = QString("%1,%2,%3,%4")
+      s1 = QString::fromLatin1("%1,%2,%3,%4")
          .arg(user->getFullName())
          .arg(user->getOffice())
          .arg(user->getWorkPhone())
          .arg(user->getHomePhone());
 #else
 
-      s = QString("%1:%2:%3:%4:")
+      s = QString::fromLatin1("%1:%2:%3:%4:")
         .arg(user->getName())
         .arg(user->getPwd())
 	.arg(user->getUID())
         .arg(user->getGID());
 
-      s1 = QString("%1,%2,%3,%4")
+      s1 = QString::fromLatin1("%1,%2,%3,%4")
          .arg(user->getFullName())
 	 .arg(user->getOffice1())
 	 .arg(user->getOffice2())
@@ -762,46 +820,48 @@ bool KUsers::savepwd() {
         s1.truncate(j);
       }
 
-      s += s1+":"+user->getHomeDir()+":"+user->getShell()+"\n";
+      s += s1+QString::fromLatin1(":")+
+           user->getHomeDir()+QString::fromLatin1(":")+
+           user->getShell()+QString::fromLatin1("\n");
 
       if( (nispasswd_fd != 0) && (minuid != 0) ) {
         if (minuid <= tmp_uid) {
-      	  fputs(QFile::encodeName(s), nispasswd_fd);	
+      	  fputs(s.local8Bit().data(), nispasswd_fd);	
           nis_users_written++;		
           continue;				
         }
         else{
-      	  fputs(QFile::encodeName(s), passwd_fd);	
+      	  fputs(s.local8Bit().data(), passwd_fd);	
           continue;				
         }
       }
 
       if( (nispasswd_fd != 0) && (minuid == 0) ) {
 	errors_found = errors_found | NOMINUID;  
-      	fputs(QFile::encodeName(s), passwd_fd);	
+      	fputs(s.local8Bit().data(), passwd_fd);	
         continue;
       }
 
       if( (nispasswd_fd == 0) && (minuid != 0) ) {
 	errors_found = errors_found | NONISPASSWD;	
-      	fputs(QFile::encodeName(s), passwd_fd);	
+      	fputs(s.local8Bit().data(), passwd_fd);	
         continue;
       }
 
-      fputs(QFile::encodeName(s), passwd_fd);	
+      fputs(s.local8Bit().data(), passwd_fd);	
 
     }		/* end for i loop */
 
     if(passwd_fd) {					
       fclose(passwd_fd);
-      chmod(QFile::encodeName(passwd_filename), pwd_mode);	
-      chown(QFile::encodeName(passwd_filename), pwd_uid, pwd_gid);	
+      chmod(pw_filename, pwd_mode);	
+      chown(pw_filename, pwd_uid, pwd_gid);	
     }							
 
     if(nispasswd_fd) {					
       fclose(nispasswd_fd);				
-      chmod(QFile::encodeName(nispasswd_filename), pwd_mode);	
-      chown(QFile::encodeName(nispasswd_filename), pwd_uid, pwd_gid);	
+      chmod(pn_filename, pwd_mode);	
+      chown(pn_filename, pwd_uid, pwd_gid);	
     }							
 
     if( (errors_found & NOMINUID) != 0 ) {	
@@ -842,27 +902,28 @@ bool KUsers::savesdw() {
   struct spwd *spwp;
   struct spwd s;
   KUser *up;
+  QString shadow_file = QString::fromLatin1(SHADOW_FILE);
 
   if (!is_shadow)
     return TRUE;
 
   if (!s_backuped) {
-    backup(SHADOW_FILE);
+    backup(shadow_file);
     s_backuped = TRUE;
   }
 
   umask(0077);
 
   if ((f = fopen(SHADOW_FILE, "w")) == NULL) {
-    err->addMsg(i18n("Error opening %1 for writing").arg(SHADOW_FILE));
+    err->addMsg(i18n("Error opening %1 for writing").arg(shadow_file));
     return FALSE;
   }
 
   s.sp_namp = (char *)malloc(200);
   s.sp_pwdp = (char *)malloc(200);
     
-  for (uint index = 0; index < u.count(); index++) {
-    up = u.at(index);
+  for (uint index = 0; index < allUsers.count(); index++) {
+    up = allUsers.at(index);
     if (up->getSPwd().isNull()) {
       err->addMsg(i18n("No shadow entry for %1.").arg(up->getName()));
       continue;
@@ -894,20 +955,20 @@ bool KUsers::savesdw() {
 }
 
 KUser *KUsers::lookup(const QString & name) {
-  for (uint i = 0; i<u.count(); i++)
-    if (u.at(i)->getName() == name)
-      return (u.at(i));
+  for (uint i = 0; i<allUsers.count(); i++)
+    if (allUsers.at(i)->getName() == name)
+      return (allUsers.at(i));
   return NULL;
 }
 
 KUser *KUsers::lookup(uid_t uid) {
-  for (uint i = 0; i<u.count(); i++)
-    if (u.at(i)->getUID() == uid)
-      return (u.at(i));
+  for (uint i = 0; i<allUsers.count(); i++)
+    if (allUsers.at(i)->getUID() == uid)
+      return (allUsers.at(i));
   return NULL;
 }
 
-uid_t KUsers::first_free() {
+int KUsers::first_free() {
   uid_t t = _KU_FIRST_UID ;
 
   for (t = _KU_FIRST_UID ; t<65534; t++)
@@ -919,80 +980,117 @@ uid_t KUsers::first_free() {
 }
 
 KUsers::~KUsers() {
-  u.clear();
-  du.clear();
+  allUsers.clear();
+  usersToDelete.clear();
 }
 
 uint KUsers::count() const {
-  return u.count();
+  return allUsers.count();
 }
 
 KUser *KUsers::operator[](uint num) {
-  return u.at(num);
+  return allUsers.at(num);
 }
 
 KUser *KUsers::first() {
-  return u.first();
+  return allUsers.first();
 }
 
 KUser *KUsers::next() {
-  return u.next();
+  return allUsers.next();
 }
 
 void KUsers::add(KUser *ku) {
-  u.append(ku);
+  allUsers.append(ku);
 }
 
-void KUsers::del(KUser *au) {
+void KUsers::del(KUser *au, bool deleteHome, bool deleteMailBox) {
   KUser *nu = new KUser(au);
-  du.append(nu);
-  u.remove(au);
+  nu->isDeleteHome = deleteHome;
+  nu->isDeleteMailBox = deleteMailBox;
+  usersToDelete.append(nu);
+  allUsers.remove(au);
 }
 
-void KUser::createHome() {
+int KUser::createHome() {
 
-  if (mkdir(QFile::encodeName(p_dir), 0700) != 0) {
-    err->addMsg(i18n("Cannot create home directory %1\nError: %2").arg(p_dir).arg(QString::fromLocal8Bit(strerror(errno))));
+  if(p_dir.isNull() || p_dir.isEmpty()) {
+    err->addMsg(i18n("Cannot create home folder for %1: it is null or empty\n").arg(p_name));
     err->display();
-    return;
+    return(0);
+  }
+  if (mkdir(QFile::encodeName(p_dir), 0700) != 0) {
+    if (errno != EEXIST)
+    {
+      err->addMsg(i18n("Cannot create home folder %1\nError: %2").arg(p_dir).arg(QString::fromLocal8Bit(strerror(errno))));
+      err->display();
+      return(0);
+    }
   }
 
   if (chown(QFile::encodeName(p_dir), p_uid, p_gid) != 0) {
-    err->addMsg(i18n("Cannot change owner of home directory\nError: %1").arg(QString::fromLocal8Bit(strerror(errno))));
+    err->addMsg(i18n("Cannot change owner of home folder %1\nError: %2").arg(p_dir).arg(QString::fromLocal8Bit(strerror(errno))));
     err->display();
-    return;
+    return(1);
   }
 
   if (chmod(QFile::encodeName(p_dir), KU_HOMEDIR_PERM) != 0) {
-    err->addMsg(i18n("Cannot change permissions on home directory\nError: %1").arg(QString::fromLocal8Bit(strerror(errno))));
+    err->addMsg(i18n("Cannot change permissions on home folder %1\nError: %2").arg(p_dir).arg(QString::fromLocal8Bit(strerror(errno))));
     err->display();
-    return;
+    return(1);
   }
+  return(1);
 }
 
 int KUser::createKDE() {
 
-	const QStringList levels = QStringList() << "/.kde/" << "share/" << "doc/";	
+	QStringList levels;	
 	QStringList types;		
 	QString k_dir = p_dir;
-	KStandardDirs kstddirs;	
+	KStandardDirs kstddirs;
+	const char *KDEHOME = "KDEHOME";
+	const char *DOT_KDE = "/.kde";
+	const char *kdehome;
+	const char *dot_kde = NULL;
 
-	for (uint level=0; level<levels.count(); level++) {	/* build kde base dirs */
+
+	if((kdehome = getenv(KDEHOME)) != NULL) {
+	    dot_kde = strstr(kdehome,DOT_KDE);
+	    if(dot_kde) {
+	        levels.append(QFile::decodeName(dot_kde));
+	        levels.append(QFile::decodeName("/share"));
+	        levels.append(QFile::decodeName("/doc"));
+	    }
+	    else {
+	        levels.append(QFile::decodeName("/.kde"));
+	        levels.append(QFile::decodeName("/share"));
+	        levels.append(QFile::decodeName("/doc"));
+	    }
+	}
+	else {
+	    levels.append(QFile::decodeName("/.kde"));
+	    levels.append(QFile::decodeName("/share"));
+	    levels.append(QFile::decodeName("/doc"));
+	}
+
+	for (uint level=0; level<levels.count(); level++) {
 		k_dir.append(levels[level]);	
 		if (tryCreate(k_dir))
 			return(-1);
  	}
 
-  types = kstddirs.KStandardDirs::allTypes();		/* retrieve kde deflt types */
+	types = kstddirs.KStandardDirs::allTypes();
 
-	for(uint i=0; i<types.count(); i++) {					/* build kde subdirectories */
-		k_dir = p_dir;															/* start with $HOME dir			*/
-		k_dir.append(levels[0]);										/* add base dir "/.kde/"		*/
-		const char *ctype = types[i].latin1();			/* convert to char for call	*/
-  	QString tpath = KStandardDirs::kde_default(ctype);		/* get deflt path */
-		k_dir.append(tpath);												/* complete full path name	*/
-		if (tryCreate(k_dir))												/* able to create it?				*/
-			return(-1);																/* -no, exit stage left			*/
+	for(uint i=0; i<types.count(); i++) {
+		k_dir = p_dir;
+		k_dir.append(levels[0]);
+		// if(dot_kde)
+		k_dir.append(QString::fromLatin1("/"));
+		const char *ctype = types[i].latin1();
+		QString tpath = KStandardDirs::kde_default(ctype);
+		k_dir.append(tpath);
+		if (tryCreate(k_dir))
+			return(-1);
 	}
 
   return(0);
@@ -1003,77 +1101,104 @@ int KUser::tryCreate(const QString &dir) {
 struct stat sb;
 int	rc = 0;
 
-rc = stat(QFile::encodeName(dir), &sb);
-	if (rc == 0) {                                      /* dir exists */
-   	if (S_ISDIR(sb.st_mode)) {												/* is directory?	*/
-																					/* yes it is...		*/
-			if (KMessageBox::
-				warningContinueCancel(0, i18n("Directory %1 already exists!\nWill make %2 owner and change permissions.\nDo you want to continue?")
+    rc = stat(QFile::encodeName(dir), &sb);
+    if (rc == 0) {
+   	if (S_ISDIR(sb.st_mode)) {
+	    if (KMessageBox::
+		warningContinueCancel(0, i18n("Folder %1 already exists!\nWill make %2 owner and change permissions.\nDo you want to continue?")
 .arg(dir).arg(p_name), QString::null, i18n("&Continue")) ==
 KMessageBox::Continue) {
-																											/* user continued */
- 				if (chown(QFile::encodeName(dir), p_uid, p_gid) != 0) {
-   				err->addMsg(i18n("Cannot change owner of %1 directory\nError: %2")
-						.arg(dir).arg(strerror(errno)));
-   				err->display();
- 				}
-		  	if (chmod(QFile::encodeName(dir), KU_KDEDIRS_PERM) != 0) {
-   				err->addMsg(i18n("Cannot change permissions on %1 directory\nError: %2").arg(dir).arg(strerror(errno)));
-   				err->display();
- 				}
-				return(0);
-			}
-			else {																					/* user cancelled	*/
-   			err->addMsg(i18n("Directory %1 left 'as is'.\nVerify ownership and permissions for user %2 who may not be able to log in!").arg(dir).arg(p_name));
-   			err->display();
-				return(-1);
-			}
-		}
-		else {																			/* exists but not as dir */
-   		err->addMsg(i18n("%1 exists and is not a directory. User %2 will not be able to log in!").arg(dir).arg(p_name));
-   		err->display();
-			return(-1);
-		}
-	}
-	else {																					/* stat rc = -1					*/
-		if (errno == ENOENT) {										/* good! dir doesn't exist	*/
- 			if (mkdir(QFile::encodeName(dir), 0700) != 0) {
-   			err->addMsg(i18n("Cannot create %1 directory\nError: %2").arg(dir)
-					.arg(strerror(errno)));
-   			err->display();
-				return(-1);
- 		 	}
- 			if (chown(QFile::encodeName(dir), p_uid, p_gid) != 0) {
-   			err->addMsg(i18n("Cannot change owner of %1 directory\nError: %2")
-					.arg(dir).arg(strerror(errno)));
-   			err->display();
- 			}
- 			if (chmod(QFile::encodeName(dir), KU_KDEDIRS_PERM) != 0) {
-   			err->addMsg(i18n("Cannot change permissions on %1 directory\nError: %2")					.arg(dir).arg(strerror(errno)));
-   			err->display();
- 			}
-			return(0);
+ 		if (chown(QFile::encodeName(dir), p_uid, p_gid) != 0) {
+   		    err->addMsg(i18n("Cannot change owner of %1 folder\nError: %2")
+.arg(dir).arg(QString::fromLocal8Bit(strerror(errno))));
+		    err->display();
  		}
-		else {																		/* some other error on stat   */
-   		err->addMsg(i18n("stat call on %1 failed.\nError: %2").arg(dir)
-				.arg(strerror(errno)));
+	  	if (chmod(QFile::encodeName(dir), KU_KDEDIRS_PERM) != 0) {
+   			err->addMsg(i18n("Cannot change permissions on %1 folder\nError: %2").arg(dir).arg(QString::fromLocal8Bit(strerror(errno))));
+   			err->display();
+ 		}
+			return(0);
+			}
+	    else {
+   		err->addMsg(i18n("Folder %1 left 'as is'.\nVerify ownership and permissions for user %2 who may not be able to log in!").arg(dir).arg(p_name));
    		err->display();
-			return(-1);
-		}
+		return(-1);
+	    }
+	}
+	else {
+   	    err->addMsg(i18n("%1 exists and is not a folder. User %2 will not be able to log in!").arg(dir).arg(p_name));
+   	    err->display();
+	    return(-1);
+	}
+    }
+    else {
+	if (errno == ENOENT) {
+ 	    if (mkdir(QFile::encodeName(dir), 0700) != 0) {
+   		err->addMsg(i18n("Cannot create %1 folder\nError: %2").arg(dir)
+		    .arg(QString::fromLocal8Bit(strerror(errno))));
+   		err->display();
+		return(-1);
+  	    }
+	    if (chown(QFile::encodeName(dir), p_uid, p_gid) != 0) {
+   		err->addMsg(i18n("Cannot change owner of %1 folder\nError: %2")
+		    .arg(dir).arg(QString::fromLocal8Bit(strerror(errno))));
+   		err->display();
+ 	    }
+	    if (chmod(QFile::encodeName(dir), KU_KDEDIRS_PERM) != 0) {
+   		err->addMsg(i18n("Cannot change permissions on %1 folder\nError: %2").arg(dir).arg(QString::fromLocal8Bit(strerror(errno))));
+		err->display();
+ 	    }
+	    return(0);
  	}
+	else {
+   	    err->addMsg(i18n("stat call on %1 failed.\nError: %2").arg(dir)
+		.arg(QString::fromLocal8Bit(strerror(errno))));
+	    err->display();
+	    return(-1);
+	}
+    }
+}
+
+bool KUser::findKDE(const QString &dir) {
+   int		kde_count     = 0;
+   const QFileInfo	*fi   = NULL;
+   const QFileInfoList	*list = NULL;
+   QDir		t(dir);
+   QString	dot_KDE = QString::fromLatin1(".kde");
+   bool		foundKDE;
+
+   foundKDE = false;
+   t.setFilter( QDir::Dirs | QDir::Hidden );
+   list = t.entryInfoList();
+   QFileInfoListIterator it( *list );
+
+   while ( (fi = it.current()) != 0 ) {
+      kde_count = fi->fileName().contains(dot_KDE, TRUE);
+      if(kde_count > 0)
+      foundKDE = true;
+      ++it;
+   }
+
+   if(foundKDE)
+	return TRUE;
+   else
+	return FALSE;
 }
 
 int KUser::createMailBox() {
   QString mailboxpath;
   int fd;
-  mailboxpath = QString("%1/%2").arg(MAIL_SPOOL_DIR).arg(p_name);
+  mailboxpath = QString::fromLatin1("%1/%2").arg(QFile::decodeName(MAIL_SPOOL_DIR)).arg(p_name);
   if((fd = open(QFile::encodeName(mailboxpath), O_CREAT|O_EXCL|O_WRONLY,
                 S_IRUSR|S_IWUSR)) < 0) {
-    err->addMsg(i18n("Cannot create %1: %2")
+    if (errno != EEXIST)
+    {
+      err->addMsg(i18n("Cannot create %1: %2")
                 .arg(mailboxpath)
                 .arg(QString::fromLocal8Bit(strerror(errno))));
-    err->display();
-    return -1;
+      err->display();
+      return -1;
+    }
   }
 
   close(fd);
@@ -1099,17 +1224,20 @@ void KUser::copyDir(const QString &srcPath, const QString &dstPath) {
   struct stat st;
   QDir s(srcPath);
   QDir d(dstPath);
-  QString prefix(SKEL_FILE_PREFIX);
+  QString prefix = QFile::decodeName(SKEL_FILE_PREFIX);
   int len = prefix.length();
 
   s.setFilter(QDir::Dirs | QDir::Hidden);
 
+  QString dot = QString::fromLatin1(".");
+  QString dotdot = QString::fromLatin1("..");
+
   for (uint i=0; i<s.count(); i++) {
     QString name(s[i]);
 
-    if (name == ".")
+    if (name == dot)
       continue;
-    if (name == "..")
+    if (name == dotdot)
       continue;
 
     QString filename(s.filePath(name));
@@ -1124,13 +1252,13 @@ void KUser::copyDir(const QString &srcPath, const QString &dstPath) {
     d.mkdir(name, FALSE);
 
     if (chown(QFile::encodeName(d.filePath(name)), p_uid, p_gid) != 0) {
-      err->addMsg(i18n("Cannot change owner of directory %1\nError: %2")
+      err->addMsg(i18n("Cannot change owner of folder %1\nError: %2")
                   .arg(d.filePath(s[i])).arg(QString::fromLocal8Bit(strerror(errno))));
       err->display();
     }
 
     if (chmod(QFile::encodeName(d.filePath(name)), st.st_mode & 07777) != 0) {
-      err->addMsg(i18n("Cannot change permissions on directory %1\nError: %2")
+      err->addMsg(i18n("Cannot change permissions on folder %1\nError: %2")
                   .arg(d.filePath(s[i])).arg(QString::fromLocal8Bit(strerror(errno))));
       err->display();
     }
@@ -1169,19 +1297,19 @@ void KUser::copyDir(const QString &srcPath, const QString &dstPath) {
 }
 
 int KUser::copySkel() {
-  QDir s(SKELDIR);
+  QDir s(QFile::decodeName(SKELDIR));
   QDir d(p_dir);
 
   umask(0777);
 
   if (!s.exists()) {
-    err->addMsg(i18n("Directory %1 does not exist").arg(s.dirName()));
+    err->addMsg(i18n("Folder %1 does not exist, cannot copy skeleton for %2").arg(s.absPath()).arg(p_name));
     err->display();
     return (-1);
   }
 
   if (!d.exists()) {
-    err->addMsg(i18n("Directory %1 does not exist").arg(d.dirName()));
+    err->addMsg(i18n("Folder %1 does not exist, cannot copy skeleton").arg(d.absPath()));
     err->display();
     return (-1);
   }
@@ -1202,22 +1330,22 @@ int KUser::removeHome() {
   if (!stat(QFile::encodeName(p_dir), &sb))
     if (S_ISDIR(sb.st_mode) && sb.st_uid == p_uid) {
 #ifdef MINIX
-      command = QString("/usr/bin/rm -rf -- %1").arg(p_dir);
+      command = QString::fromLatin1("/usr/bin/rm -rf -- %1").arg(KProcess::quote(p_dir));
 #else
-      command = QString("/bin/rm -rf -- %1").arg(p_dir);
+      command = QString::fromLatin1("/bin/rm -rf -- %1").arg(KProcess::quote(p_dir));
 #endif
       if (system(QFile::encodeName(command)) != 0) {
-             err->addMsg(i18n("Cannot remove home directory %1\nError: %2")
-                       .arg(command).arg(strerror(errno)));
+             err->addMsg(i18n("Cannot remove home folder %1\nError: %2")
+                       .arg(command).arg(QString::fromLocal8Bit(strerror(errno))));
              err->display();
       }
     } else {
-      err->addMsg(i18n("Removal of home directory %1 failed (uid = %2, gid = %3)").arg(p_dir).arg(sb.st_uid).arg(sb.st_gid));
+      err->addMsg(i18n("Removal of home folder %1 failed (uid = %2, gid = %3)").arg(p_dir).arg(sb.st_uid).arg(sb.st_gid));
       err->display();
     }
   else {
     err->addMsg(i18n("stat call on file %1 failed\nError: %2")
-                 .arg(p_dir).arg(strerror(errno)));
+                 .arg(p_dir).arg(QString::fromLocal8Bit(strerror(errno))));
     err->display();
   }
 
@@ -1230,9 +1358,9 @@ int KUser::removeCrontabs() {
   QString file;
   QString command;
 
-  file = QString("/var/cron/tabs/%1").arg(p_name);
+  file = QString::fromLatin1("/var/cron/tabs/%1").arg(p_name);
   if (access(QFile::encodeName(file), F_OK) == 0) {
-    command = QString("crontab -u %1 -r").arg(p_name);
+    command = QString::fromLatin1("crontab -u %1 -r").arg(KProcess::quote(p_name));
     if (system(QFile::encodeName(command)) != 0) {
       err->addMsg(i18n("Cannot remove crontab %1\nError: %2")
                   .arg(command).arg(QString::fromLocal8Bit(strerror(errno))));
@@ -1246,7 +1374,7 @@ int KUser::removeCrontabs() {
 int KUser::removeMailBox() {
   QString file;
 
-  file = QString("%1/%2").arg(MAIL_SPOOL_DIR).arg(p_name);
+  file = QString::fromLatin1("%1/%2").arg(QFile::decodeName(MAIL_SPOOL_DIR)).arg(p_name);
   if (remove(QFile::encodeName(file)) != 0) {
     err->addMsg(i18n("Cannot remove mailbox %1\nError: %2")
                 .arg(file).arg(QString::fromLocal8Bit(strerror(errno))));
